package footyquiz.server.integration;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.*;
import javax.sql.*;

import footyquiz.server.tools.Log;

/**
 * Database Connector connects to the database using connection pooling. 
 * @author Mikael Kopteff & Hannu Leinonen
 * 
 */
public class DatabaseConnector {
	private Connection connection = null;

	/**
	 * Default constructor, that uses dbname in setup file
	 */
	public DatabaseConnector() {
	}

	/**
	 * Constructor creates an instance with the connection passed to it. When
	 * connection passed to it, createConnection() -method is obsolete.
	 * 
	 * @param connection -
	 *            Connection used to connect to database
	 */
	public DatabaseConnector(Connection connection) {
		this.connection = connection;
	}

	/**
	 * Connects to the database. Return true if connection created succefully.
	 * 
	 * @return true if successful
	 */
	public boolean createConnection() {
		try {
			Context context = new InitialContext();
			Context envelope = (Context) context.lookup("java:comp/env");
			DataSource datasource = (DataSource) envelope
					.lookup("jdbc/dbname");
			connection = datasource.getConnection();
		}
		catch (Exception ex) {
			Log.logger.severe("DB error/create Connection :" + ex.getMessage());
			ex.printStackTrace();
			return (false);
		}
		return (true);
	}

	/**
	 * Kills the database connection. Returns true if connection closed
	 * succesfully.
	 * 
	 * @return true if successful
	 */
	public boolean closeConnection() {
		try {
			connection.close();
		}
		catch (SQLException ex) {
			Log.logger.severe("DB error/Close connection :" + ex.getMessage());
			ex.printStackTrace();
			return (false);
		}
		return (true);
	}

	/**
	 * Executes SQL quary using the sql string pased to it. If autoconnect is
	 * true method opens db itself.
	 * 
	 * @param sql -
	 *            SQL to execute
	 * @param autoconnect -
	 *            If true, database connection will be opened
	 * @return - true is execution succesfull
	 */
	public boolean executeSQL(String sql, List<String> values, boolean autoconnect) {
		try {
			if(autoconnect) {
				createConnection();
			}
			PreparedStatement s = connection.prepareStatement(sql);
			Log.logger.info("Running SQL:" + sql);
			for(int i = 0; i < values.size(); i++) {
				s.setObject(i + 1, values.get(i).replaceAll("'", "''"));
			}
			s.execute();
			if(autoconnect) {
				closeConnection();
			}
		}
		catch (SQLException ex) {
			Log.logger.info("SQL integrity violation occurred :"
					+ ex.getMessage());
			return (false);
		}
		catch (Exception ex) {
			Log.logger.info("DB error/ExecuteSQL " + ex.getMessage());
			ex.printStackTrace();// Connection/Disconnection/General error
			return (false);
		}
		return (true);
	}

	/**
	 * Sets the transaction control to the database.
	 * 
	 * @param value
	 *            if true, database autocommits
	 */
	public void setAutocommit(boolean value) {
		if(connection != null) {
			try {
				connection.setAutoCommit(value);
			}
			catch (SQLException ex) {
				Log.logger.info("DB error/Autocommit :" + ex.getMessage());
				ex.printStackTrace();
			}
		}
	}

	/**
	 * This method commits the data to the database if transaction is set to
	 * true.
	 */
	public boolean commit() {
		if(connection != null) {
			try {
				connection.commit();
			}
			catch (SQLException ex) {
				Log.logger.severe("DB error/commit " + ex.getMessage());
				ex.printStackTrace();
				return false;
			}
		}
		return true;
	}

	/**
	 * This method rolls back if transaction is set to true.
	 * 
	 * @return true if succesful
	 */
	public boolean rollback() {
		if(connection != null) {
			try {
				connection.rollback();
			}
			catch (SQLException ex) {
				Log.logger.severe("DB error/rollback " + ex.getMessage());
				ex.printStackTrace();
				return false;
			}
		}
		return true;
	}

	/**
	 * SQL queries with where conditions should be run here. The list of values
	 * to be out in the Statement is given as parameter.
	 * 
	 * @param sql
	 * @param values
	 * @return A List of HashMaps where the results can be found.
	 */
	public List<Map> executeQuery(String sql, List<String> values, boolean autoconnect) {
		Log.logger.info("Executing query: " + sql);
		List<Map> resultSet = null;
		try {
			if(autoconnect) {
				createConnection();
			}
			resultSet = new ArrayList<Map>();
			PreparedStatement s = connection.prepareStatement(sql);
			for(int i = 0; i < values.size(); i++) {
				s.setObject(i + 1, values.get(i).replaceAll("'", "''"));
			}
			ResultSet rs = s.executeQuery();
			while (rs.next()) {
				Map<String, String> m = new HashMap<String, String>();
				String key = "";
				String value = "";
				for(int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
					key = rs.getMetaData().getColumnLabel(i);
					value = rs.getString(i);
					if(value != null)
						value = value.replaceAll("''", "'");
					m.put(key, value);
				}
				resultSet.add(m);
			}
			rs.close();
			s.close();
			if(autoconnect) {
				closeConnection();
			}
		}
		catch (SQLException e) {
			Log.logger.severe(e.getMessage());
			e.printStackTrace();
		}
		return resultSet;
	}

	/**
	 * Executes sql, that wil return the results in an Arraylist. Methods
	 * breaks-down the resultset into HashMap in an arraylist.
	 * 
	 * @param sql -
	 *            Query to execute
	 * @param autoconnect -
	 *            if true, connects db automatically
	 * @return - ArrayList values from query
	 */
	public List<Map> executeQuery(String sql, boolean autoconnect) {
		Log.logger.info("Executing query: " + sql);
		List<Map> resultSet = null;
		if(autoconnect) {
			createConnection();
		}
		resultSet = new ArrayList<Map>();
		try {
			Statement s = connection.createStatement();
			ResultSet rs = s.executeQuery(sql);
			while (rs.next()) {
				Map<String, String> m = new HashMap<String, String>();
				String key = "";
				String value = "";
				for(int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
					key = rs.getMetaData().getColumnLabel(i);
					value = rs.getString(i);
					if(value != null)
						value = value.replaceAll("''", "'");
					m.put(key, value);
				}
				resultSet.add(m);
			}
			rs.close();
			s.close();
			if(autoconnect) {
				closeConnection();
			}
		}
		catch (SQLException ex) {
			Log.logger.severe("DB error/executeQuery " + ex.getMessage());
			ex.printStackTrace();
		}

		return resultSet;
	}
}
